@@ -1,4 +1,4 @@
-# Copyright (c) 2014, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -75,14 +75,16 @@ try:
         CMySQLCursorNamedTuple,
         CMySQLCursorPrepared,
         CMySQLCursorPreparedDict,
+        CMySQLCursorPreparedNamedTuple,
+        CMySQLCursorPreparedRaw,
         CMySQLCursorRaw,
     )
+
+    HAVE_CMYSQL = True
 except ImportError as exc:
     raise ImportError(
         f"MySQL Connector/Python C Extension not available ({exc})"
     ) from exc
-else:
-    HAVE_CMYSQL = True
 
 
 class CMySQLConnection(MySQLConnectionAbstract):
@@ -247,6 +249,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
             "local_infile": self._allow_local_infile,
             "load_data_local_dir": self._allow_local_infile_in_path,
             "oci_config_file": self._oci_config_file,
+            "oci_config_profile": self._oci_config_profile,
             "fido_callback": self._fido_callback,
         }
 
@@ -705,7 +708,9 @@ class CMySQLConnection(MySQLConnectionAbstract):
             8: CMySQLCursorNamedTuple,
             9: CMySQLCursorBufferedNamedTuple,
             16: CMySQLCursorPrepared,
+            18: CMySQLCursorPreparedRaw,
             20: CMySQLCursorPreparedDict,
+            24: CMySQLCursorPreparedNamedTuple,
         }
         try:
             return (types[cursor_type])(self)
@@ -765,7 +770,9 @@ class CMySQLConnection(MySQLConnectionAbstract):
             if self.converter:
                 result = [
                     self.converter.quote(
-                        self.converter.escape(self.converter.to_mysql(value))
+                        self.converter.escape(
+                            self.converter.to_mysql(value), self._sql_mode
+                        )
                     )
                     for value in params
                 ]
@@ -776,7 +783,9 @@ class CMySQLConnection(MySQLConnectionAbstract):
             if self.converter:
                 for key, value in params.items():
                     result[key] = self.converter.quote(
-                        self.converter.escape(self.converter.to_mysql(value))
+                        self.converter.escape(
+                            self.converter.to_mysql(value), self._sql_mode
+                        )
                     )
             else:
                 for key, value in params.items():
@@ -806,6 +815,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
         password2: str = "",
         password3: str = "",
         oci_config_file: Optional[str] = None,
+        oci_config_profile: Optional[str] = None,
     ) -> None:
         """Change the current logged in user"""
         try:
@@ -817,6 +827,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
                 password2,
                 password3,
                 oci_config_file,
+                oci_config_profile,
             )
 
         except MySQLInterfaceError as err:
@@ -954,6 +965,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
                     self._password2,
                     self._password3,
                     self._oci_config_file,
+                    self._oci_config_profile,
                 )
             except ProgrammingError:
                 self.reconnect()
