@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2022, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2023, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -31,7 +31,7 @@
 import warnings
 
 from abc import ABC, ABCMeta
-from typing import Dict, List, Optional, Sequence, Tuple, Union, ValuesView
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union, ValuesView
 
 from .charsets import MYSQL_CHARACTER_SETS, MYSQL_CHARACTER_SETS_57
 from .errors import ProgrammingError
@@ -90,6 +90,7 @@ DEFAULT_CONFIGURATION: Dict[str, Optional[Union[str, bool, int]]] = {
     "use_pure": False,
     "krb_service_principal": None,
     "oci_config_file": None,
+    "oci_config_profile": None,
     "fido_callback": None,
     "kerberos_auth_mode": None,
     "init_command": None,
@@ -112,8 +113,18 @@ def flag_is_set(flag: int, flags: int) -> bool:
 
 
 def _obsolete_option(name: str, new_name: str, value: int) -> int:
+    """Raise a deprecation warning and advise a new option name.
+
+    Args:
+        name (str): The name of the option.
+        new_name (str): The new option name.
+        value (int): The value of the option.
+
+    Returns:
+        int: The value of the option.
+    """
     warnings.warn(
-        f'The option "{name}" has been deprecated, use "{new_name}" instead.',
+        f"The option '{name}' has been deprecated, use '{new_name}' instead.",
         category=DeprecationWarning,
     )
     return value
@@ -339,7 +350,27 @@ class FieldFlag(_Flags):
     }
 
 
-class ServerCmd(_Constants):
+class ServerCmdMeta(ABCMeta):
+    """ClientFlag Metaclass."""
+
+    def __getattribute__(cls, name: str) -> Any:
+        deprecated_options = (
+            "FIELD_LIST",
+            "REFRESH",
+            "SHUTDOWN",
+            "PROCESS_INFO",
+            "PROCESS_KILL",
+        )
+        if name in deprecated_options:
+            warnings.warn(
+                f"The option 'ServerCmd.{name}' is deprecated and will be removed in "
+                "a future release.",
+                category=DeprecationWarning,
+            )
+        return super().__getattribute__(name)
+
+
+class ServerCmd(_Constants, metaclass=ServerCmdMeta):
     """MySQL Server Commands"""
 
     _prefix: str = "COM_"
@@ -441,7 +472,8 @@ class ClientFlag(_Flags):
     CONNECT_ARGS: int = 1 << 20
     PLUGIN_AUTH_LENENC_CLIENT_DATA: int = 1 << 21
     CAN_HANDLE_EXPIRED_PASSWORDS: int = 1 << 22
-    SESION_TRACK: int = 1 << 23
+    SESION_TRACK: int = 1 << 23  # deprecated
+    SESSION_TRACK: int = 1 << 23
     DEPRECATE_EOF: int = 1 << 24
     CLIENT_QUERY_ATTRIBUTES: int = 1 << 27
     SSL_VERIFY_SERVER_CERT: int = 1 << 30
@@ -478,7 +510,11 @@ class ClientFlag(_Flags):
             1 << 22,
             "Don't close the connection for a connection with expired password",
         ),
-        "SESION_TRACK": (
+        "SESION_TRACK": (  # deprecated
+            1 << 23,
+            "Capable of handling server state change information",
+        ),
+        "SESSION_TRACK": (
             1 << 23,
             "Capable of handling server state change information",
         ),
@@ -740,8 +776,10 @@ class CharacterSet(_Constants):
             try:
                 info = cls.desc[charset]
                 return info[1], info[0], charset
-            except (IndexError, KeyError):
-                ProgrammingError(f"Character set ID '{charset}' unsupported")
+            except (IndexError, KeyError) as err:
+                raise ProgrammingError(
+                    f"Character set ID '{charset}' unsupported"
+                ) from err
 
         for cid, info in enumerate(cls.desc):
             if info is None:
@@ -775,8 +813,8 @@ class CharacterSet(_Constants):
             try:
                 info = cls.desc[charset]
                 return (charset, info[0], info[1])
-            except IndexError:
-                ProgrammingError(f"Character set ID {charset} unknown")
+            except IndexError as err:
+                raise ProgrammingError(f"Character set ID {charset} unknown") from err
 
         if charset in ("utf8", "utf-8") and cls.mysql_version == (8, 0):
             charset = "utf8mb4"
